;************************************************
;Programa : LCDIII.asm
;Data     : 29/05/2012
;Autor    : Prof. Vargas
;Descricao:
;Acionar um LCD com 8 bits de dados com mensagens
;diferentes e modos diferentes do padrao
;************************************************

;Definicoes			
;**********
#INCLUDE <P16F628A.INC>

BANK0	MACRO				;Macro seleciona BANK0
			bcf	STATUS,RP0
			bcf	STATUS,RP1
		ENDM
BANK1	MACRO				;Macro seleciona BANK1
			bsf	STATUS,RP0
			bcf	STATUS,RP1
		ENDM

;***************
;  CONSTANTES
;***************

INIC_TMR0		EQU	.131		;inicial do TMR0
CONTA			EQU	.125		;inicial do COUNT	

#DEFINE	Buffer_size		.32		;notacao decimal
#DEFINE	END_INIC_BUFFER	0x20
#DEFINE	END_FIM_BUFFER	0x3F
#DEFINE	END_INIC_LINHA1	0x20
#DEFINE	END_FIM_LINHA1	0x2F
#DEFINE	END_INIC_LINHA2	0x30
#DEFINE	END_FIM_LINHA2	0x3F

;Definicoes do LCD
#DEFINE	E			PORTA,1		;Corresponde a RA1
#DEFINE	RS			PORTA,2		;Corresponde a RA2
;***************************
;		VARIAVEIS				
;***************************
COUNT		EQU	0x41	; ou H'41'
COUNT1		EQU	0x42	; ou H'42'...
COUNT2		EQU	0x43
min_sec		EQU	0x44
VALOR		EQU	0x45
d1			EQU	0x46
d2			EQU	0x47
d3			EQU	0x48
contador	EQU	0x49

	ORG		0x00
	goto	Inicio

	ORG		0x04
	RETFIE				;Interrupcoes off

;=============================
;    Programa Principal
;=============================
Inicio:
	BANK0
	clrf	PORTA
	movlw	0x07
	movwf	CMCON
	BANK1
	movlw	B'00000000'	
	movwf	TRISA		;PORT A so sada
	movlw	B'00000000'	
	movwf	TRISB		;PORT B so sada
	movlw	B'10000100'	;Prescaler 1:32 em TMR0
	movwf	OPTION_REG	;Pull ups desabilitados
	movlw 	B'00100000'	;Interrupcoes desligadas
	movwf	INTCON	
	BANK0

;	Inicia as variaveis
;------------------------
	clrf	PORTA
	clrf	PORTB

	movlw	CONTA
	movwf	COUNT			;Inicia COUNT com .125
	movlw	INIC_TMR0
	movwf	TMR0			;inicia TMR0 com .131

principal:
	call	INICIA_LCD
	movlw	.6
	movwf	min_sec			;Comandos para aguardar 3S
wait0_5S:
	call 	HalfSec			;Aguarda 0,5 segundo
	call	Delay_2mS		;Segura 4mS
	call	Delay_2mS
	call	ACIONA_LCD
	call	HalfSec			;aguarda 0,5S
	call	Delay_2mS		;Segura 4mS
	call	Delay_2mS
	goto	$


;*****************************************
;Rotina INICIA_LCD
;Inicializa o LCD e acende todos os pixels
;*****************************************
INICIA_LCD:
	bcf	E					;E=0 
	bcf	RS					;RS=0, Via de dados contem instrucao 
	call	Delay_50uS		;sinaliza 50 uS
	movlw	0x30			;Configura LCD: 0011.0000
	movwf	PORTB			;Inicia LCD
	call	Delay_2mS		;Segura 4mS
	call	Delay_2mS
	movlw	0x30			;Configura LCD: 0011.0000
	movwf	PORTB
	call	PULSE			;pulsa ~ 53uS
	call	PULSE
	movlw	0x30			;Configura LCD: 0011.0000
	movwf	PORTB			;Inicializa
	call	PULSE			;Pulsa ~53uS
	movlw	0x38			;0011.1000 8-bit, 2 linhas
	movwf	PORTB			;e caracteres com 5x7 
	call	PULSE
	movlw	0x0D			;0000.1101 Liga display e
	movwf	PORTB			;desliga cursor piscante
	call	PULSE
	movlw	0x06			;0000.0110 autoincremento=off
                            ;display = off 
	movwf	PORTB			;novo caractere
	call	PULSE
	movlw	0x01			;0000.0001 limpa display e 
	movwf	PORTB			;reseta contador
	call	PULSE
	call	Delay_2mS
	call	Delay_2mS
	movlw	0xFF
	movwf	VALOR			;Valor = FFh
	call	ACIONA_LCD
	return

;***************************************
;Rotina Tabela:
;Contem os dados a serem enviados ao LCD
;***************************************
Tabela:
		ADDWF		PCL,F
		retlw		' '
		retlw		' '
		retlw		'P'
		retlw		'r'
		retlw		'o'
		retlw		'f'
		retlw		' '
		retlw		'V'
		retlw		'a'
		retlw		'r'
		retlw		'g'
		retlw		'a'
		retlw		's'
		retlw		' '
		retlw		' '
		retlw		' '
		retlw		' '	;inicio da 2a linha
		retlw		' '
		retlw		' '
		retlw		' '
		retlw		'T'
		retlw		'h'
		retlw		'e'
		retlw		' '
		retlw		'b'
		retlw		'e'
		retlw		's'
		retlw		't'
		retlw		' '
		retlw		' '
		retlw		' '
		retlw		' '		
;*****************************************
;Rotina ACIONA_LCD:
;Atualiza o LCD com os valores armazenado
;no Buffer (20h-3Fh)
;*****************************************
ACIONA_LCD:
	bcf		E				;Linha   E = 0
	bcf		RS				;Linha /RS = 0 LCD recebe
                            ;instrucao 
 	movlw	0x80			;1000.0000
	movwf	PORTB			;Instrucao para acesso a DDRAM
    call	PULSE			;grava no LCD
	bsf		RS				;RS=1, LCD recebe dado

	clrf	contador
OBTEM_CARACTERE:
	movf	contador,W
	call	Tabela
		
	movwf	PORTB			;Envia W para o LCD
	call	PULSE			;Grava no LCD

	incf	contador,W
	xorlw	d'16'
	btfsc	STATUS,Z
	goto	linha2
	incf	contador,F
	goto	OBTEM_CARACTERE
linha2:
	bcf		E				;Linha   E = 0
	bcf		RS				;Linha /RS = 0 LCD recebe
                            ;instrucao 
	movlw	0xC0			;altera endereco DDRAM 0x40
	movwf	PORTB
	call	PULSE			;grava LCD
	bsf		RS				;volta para o modo DADOS
	movlw	d'17'
	movwf	contador		;atualiza contador 2a linha
again:
	movf	contador,W
	call	Tabela
		
	movwf	PORTB			;Envia W para o LCD
	call	PULSE			;Grava no LCD

	incf	contador,W
	xorlw	d'32'
	btfsc	STATUS,Z
	goto	retorna
	incf	contador,F
	goto	again

retorna:
	return

;*******************************
;Rotina HalfSec:
;Temporiza 0,5 S = 500000 cycles
; Erro de 0 %
;*******************************
HalfSec:       ;499994 cycles
	movlw	0x03
	movwf	d1
	movlw	0x18
	movwf	d2
	movlw	0x02
	movwf	d3
Delay_0:
	decfsz	d1, f
	goto	$+2
	decfsz	d2, f
	goto	$+2
	decfsz	d3, f
	goto	Delay_0
			;2 cycles
	goto	$+1
			;4 cycles (including call)
	return
;*********************************
;Rotina Delay_50uS:
;Temporiza 50 uS para Fosc = 4MHz
;*********************************
Delay_50uS:
	movlw	.15
	movwf	COUNT1			;COUNT1 = 15

REPEAT:
	decfsz	COUNT1,F
	goto	REPEAT
	return
    	
;******************************
;Rotina Delay_2mS:
;Delay de 2 mS para Fosc = 4MHz
;******************************
Delay_2mS:
	movlw	.38
	movwf	COUNT2			;COUNT2 = 38
	
DELAY:
	call	Delay_50uS
	decfsz	COUNT2,F
	goto	DELAY
	return
    
;********************************
;Rotina PULSE:
;Aplica um pulso na linha do LCD.
;********************************
PULSE:
	bsf		E				;Seta linha e aguarda
	nop						;1 uS e apaga a linha
	bcf		E
	call	Delay_50uS
	return
end
